@@ -1,4 +1,5 @@
-import os, sys, multiprocessing, tempfile
+import os, sys, multiprocessing, tempfile, platform
+from mama.platforms.oclea import Oclea
 import mama.util as util
 from .utils.system import System, console
 from .utils.sub_process import execute, execute_piped
@@ -25,6 +26,10 @@ class BuildConfig:
         self.upload  = False
         # currently for uploads only, uploads only if package already not uploaded
         self.if_needed = False
+        # if `art` is specified, then artifactory download is mandatory, no source builds are done
+        self.force_artifactory = False
+        # if `noart` is specified, then artifactory is temporarily ignored
+        self.disable_artifactory = False
         self.reclone   = False
         self.mama_init = False
         self.print     = True
@@ -38,7 +43,7 @@ class BuildConfig:
         self.ios     = False
         self.android = False
         self.raspi   = False
-        self.oclea    = False
+        self.oclea : Oclea = None
         # compilers
         self.clang = True # prefer clang on linux
         self.gcc   = False
@@ -57,6 +62,7 @@ class BuildConfig:
         self.debug   = False
         # valid architectures: x86, x64, arm, arm64
         self.arch    = None
+        self.distro  = None  # distro information (name, major, minor)
         self.jobs    = multiprocessing.cpu_count()
         self.target  = None
         self.flags   = None
@@ -85,10 +91,6 @@ class BuildConfig:
         self.raspi_compilers  = ''  ## Raspberry g++ and gcc
         self.raspi_system     = ''  ## path to Raspberry system libraries
         self.raspi_include_paths = [] ## path to additional Raspberry include dirs
-        ## Oclea CV25/CVXX
-        self.oclea_compilers = ''  ## Oclea g++, gcc and ld
-        self.oclea_system    = ''  ## Path to Oclea system libraries
-        self.oclea_include_paths = []  ## Path to additional Oclea include dirs
         ## Convenient installation utils:
         self.convenient_install = []
         ## Workspace and parsing
@@ -113,6 +115,8 @@ class BuildConfig:
             elif arg == 'deploy':    self.deploy  = True
             elif arg == 'upload':    self.upload  = True
             elif arg == 'if_needed': self.if_needed = True
+            elif arg == 'art':       self.force_artifactory = True
+            elif arg == 'noart':     self.disable_artifactory = True
             # Updates, Builds and Deploys the project as a package
             elif arg == 'serve':
                 self.build = True
@@ -208,7 +212,8 @@ class BuildConfig:
         self.ios     = ios
         self.android = android
         self.raspi   = raspi
-        self.oclea   = oclea
+        if not oclea: self.oclea = None
+        elif not self.oclea: self.oclea = Oclea(self)
         return True
 
 
@@ -245,6 +250,41 @@ class BuildConfig:
                 raise RuntimeError(f'Unsupported arch={self.arch} on Oclea platform! Supported=arm64')
 
 
+    def get_distro_info(self):
+        if self.distro:
+            return self.distro
+        if self.windows:
+            version = platform.version().split('.') + ['0']
+            self.distro = (self.name(), int(version[0]), int(version[1]))
+        elif self.macos:
+            version = self.macos_version.split('.') + ['0']
+            self.distro = (self.name(), int(version[0]), int(version[1]))
+        elif self.ios:
+            version = self.ios_version.split('.') + ['0']
+            self.distro = (self.name(), int(version[0]), int(version[1]))
+        elif self.android:
+            version = self.android_api.split('-')[1]
+            self.distro = (self.name(), int(version), 0)
+        elif self.raspi:
+            # TODO: RASPI version
+            self.distro = (self.name(), 0, 0)
+        elif self.oclea:
+            # TODO: OCLEA version
+            self.distro = (self.name(), 0, 0)
+        elif self.linux:
+            try:
+                dist = distro.info()
+                major = int(dist['version_parts']['major'])
+                minor = int(dist['version_parts']['minor'])
+                self.distro = (dist['id'], major, minor)
+            except Exception as err:
+                console(f'Failed to parse linux distro; falling back to Ubuntu 16.04 LTS: {err}', color='red')
+                self.distro = ('ubuntu', 16, 4)
+        else:
+            self.distro = (platform.system().lower(), int(platform.release()), 0)
+        return self.distro
+
+
     def set_arch(self, arch):
         arches = ['x86', 'x64', 'arm', 'arm64']
         if not arch in arches:
@@ -405,11 +445,11 @@ class BuildConfig:
             ext = '.exe' if System.windows else ''
             self.cc_path  = f'{self.raspi_bin()}arm-linux-gnueabihf-gcc{ext}'
             self.cxx_path = f'{self.raspi_bin()}arm-linux-gnueabihf-g++{ext}'
-            self.cxx_version = self._get_gcc_clang_version(self.cc_path)
+            self.cxx_version = self._get_gcc_clang_fullversion(self.cc_path)
         elif self.oclea:
-            self.cc_path  = f'{self.oclea_bin()}aarch64-oclea-linux-gcc'
-            self.cxx_path = f'{self.oclea_bin()}aarch64-oclea-linux-g++'
-            self.cxx_version = self._get_gcc_clang_version(self.cc_path)
+            self.cc_path  = f'{self.oclea.bin()}aarch64-oclea-linux-gcc'
+            self.cxx_path = f'{self.oclea.bin()}aarch64-oclea-linux-g++'
+            self.cxx_version = self._get_gcc_clang_fullversion(self.cc_path)
         elif self.clang:
             suffixes = ['-12','-11','-10','-9','-8','-7','-6','']
             self.clang_path, suffix, ver = self.find_compiler_root(self.clang_path, 'clang++', suffixes)
@@ -440,9 +480,10 @@ class BuildConfig:
             return f'msvc{version}'
         elif self.linux or self.raspi or self.oclea:
             cc, _, version = self.get_preferred_compiler_paths()
-            major_version = version.split('.')[0]
-            if 'gcc' in cc: return f'gcc{major_version}'
-            if 'clang' in cc: return f'clang{major_version}'
+            version_parts = version.split('.')
+            major_version, minor_version = version_parts[0], version_parts[1]
+            if 'gcc' in cc: return f'gcc{major_version}.{minor_version}'
+            if 'clang' in cc: return f'clang{major_version}.{minor_version}'
             raise EnvironmentError(f'Unrecognized compiler {cc}!')
         elif self.macos:
             return self.macos_version
@@ -486,36 +527,6 @@ class BuildConfig:
     def android_ndk(self):
         if not self.android_ndk_path: self.init_ndk_path()
         return self.android_ndk_path
-    
-
-    def raspi_bin(self):
-        if not self.raspi_compilers: self.init_raspi_path()
-        return self.raspi_compilers
-
-
-    def raspi_sysroot(self):
-        if not self.raspi_compilers: self.init_raspi_path()
-        return self.raspi_system
-
-    
-    def raspi_includes(self):
-        if not self.raspi_compilers: self.init_raspi_path()
-        return self.raspi_include_paths
-    
-
-    def oclea_bin(self):
-        if not self.oclea_compilers: self.init_oclea_path()
-        return self.oclea_compilers
-
-
-    def oclea_sysroot(self):
-        if not self.oclea_compilers: self.init_oclea_path()
-        return self.oclea_system
-
-    
-    def oclea_includes(self):
-        if not self.oclea_compilers: self.init_oclea_path()
-        return self.oclea_include_paths
 
 
     def init_ndk_path(self):
@@ -537,6 +548,21 @@ Default search paths: {paths}
 Define env ANDROID_HOME with path to Android SDK with NDK at ${{ANDROID_HOME}}/ndk-bundle.''')
 
 
+    def raspi_bin(self):
+        if not self.raspi_compilers: self.init_raspi_path()
+        return self.raspi_compilers
+
+
+    def raspi_sysroot(self):
+        if not self.raspi_compilers: self.init_raspi_path()
+        return self.raspi_system
+
+    
+    def raspi_includes(self):
+        if not self.raspi_compilers: self.init_raspi_path()
+        return self.raspi_include_paths
+
+
     def init_raspi_path(self):
         paths = []
         self.append_env_path(paths, 'RASPI_HOME')
@@ -560,24 +586,21 @@ Default search paths: {paths}
 Define env RASPI_HOME with path to Raspberry tools.''')
 
 
-    def init_oclea_path(self):
-        if not System.linux: raise RuntimeError('Oclea only supported on Linux')
-        paths = []
-        self.append_env_path(paths, 'OCLEA_HOME')
-        self.append_env_path(paths, 'OCLEA_SDK')
-        if System.linux: paths += ['/usr/bin/oclea', '/usr/local/bin/oclea']
-        compiler = ''
-        if System.linux: compiler = 'x86_64-ocleasdk-linux/usr/bin/aarch64-oclea-linux/aarch64-oclea-linux-gcc'
-        for oclea_path in paths:
-            if os.path.exists(f'{oclea_path}/{compiler}'):
-                self.oclea_compilers = f'{oclea_path}/x86_64-ocleasdk-linux/usr/bin/aarch64-oclea-linux/'
-                self.oclea_system    = f'{oclea_path}/x86_64-ocleasdk-linux/usr/lib'
-                self.oclea_include_paths = [f'{oclea_path}/aarch64-oclea-linux/usr/include']
-                if self.print: console(f'Found Oclea TOOLS: {self.oclea_compilers}\n    sysroot: {self.oclea_system}')
-                return
-        raise EnvironmentError(f'''No Oclea toolchain compilers detected! 
-Default search paths: {paths} 
-Define env OCLEA_HOME with path to Oclea tools.''')
+    def set_oclea_toolchain(self, toolchain_dir, toolchain_file=None):
+        """
+        Sets the toolchain file where these dirs exist:
+            aarch64-oclea-linux/
+            x86_64-ocleasdk-linux/
+        And optionally also sets the CMake toolchain file via `toolchain_dir`
+        """
+        if not os.path.exists(toolchain_dir):
+            raise FileNotFoundError(f'Toolchain directory not found: {toolchain_dir}')
+        if toolchain_file:
+            if not os.path.exists(toolchain_file):
+                raise FileNotFoundError(f'Toolchain file not found: {toolchain_file}')
+            self.oclea.init_oclea_toolchain_file(toolchain_file, toolchain_dir)
+        else:
+            self.oclea.init_oclea_path(toolchain_dir)
 
 
     def find_default_fortran_compiler(self):
@@ -719,40 +742,27 @@ Define env OCLEA_HOME with path to Oclea tools.''')
     def install_clang6(self):
         if System.windows: raise OSError('Install Visual Studio 2019 with Clang support')
         if System.macos:   raise OSError('Install Xcode to get Clang on macOS')
-        
-        suffix = '1404'
-        try:
-            dist = distro.info()
-            if dist['id'] != "ubuntu": raise OSError('install_clang6 only supports Ubuntu')
-            majorVersion = int(dist['version_parts']['major'])
-            if majorVersion >= 16:
-                suffix = '1604'
-            console(f'Choosing {suffix} for kernel major={majorVersion}')
-        except Exception as err:
-            console(f'Failed to parse linux distro; falling back to {suffix}: {err}')
 
+        id, major, _ = self.get_distro_info()
+        if id != "ubuntu": raise OSError('install_clang6 only supports ubuntu')
+        suffix = '1404'
+        if major >= 16: suffix = '1604'
+
+        console(f'Choosing {suffix} for kernel major={major}')
         self.install_clang(clang_major='6', clang_ver='6.0', suffix=suffix)
 
 
     def install_clang11(self):
         if System.windows: raise OSError('Install Visual Studio 2019 with Clang support')
         if System.macos:   raise OSError('Install Xcode to get Clang on macOS')
-        
+
+        id, major, minor = self.get_distro_info()
+        if id != "ubuntu": raise OSError('install_clang11 only supports ubuntu')
         suffix = '1604'
-        try:
-            dist = distro.info()
-            if dist['id'] != "ubuntu": raise OSError('install_clang11 only supports Ubuntu')
-            majorVersion = int(dist['version_parts']['major'])
-            minorVersion = int(dist['version_parts']['minor'])
-            if majorVersion >= 20 and minorVersion >= 10:
-                suffix = '2010'
-            elif majorVersion >= 20:
-                suffix = '2004'
-            elif majorVersion >= 16:
-                suffix = '1604'
-            console(f'Choosing {suffix} for kernel major={majorVersion} minor={minorVersion}')
-        except Exception as err:
-            console(f'Failed to parse linux distro; falling back to {suffix}: {err}')
+        if major >= 20 and minor >= 10: suffix = '2010'
+        elif major >= 20: suffix = '2004'
+        elif major >= 16: suffix = '1604'
+        console(f'Choosing {suffix} for kernel major={major} minor={minor}')
 
         self.install_clang(clang_major='11', clang_ver='11.0', suffix=suffix)
 
@@ -780,10 +790,10 @@ Define env OCLEA_HOME with path to Oclea tools.''')
     def install_msbuild(self):
         if System.windows: raise OSError('Install Visual Studio 2019 to get MSBuild on Windows')
         if System.macos:   raise OSError('install_msbuild not implemented for macOS')
-        
-        dist = distro.info()
-        if dist['id'] != "ubuntu": raise OSError('install_msbuild only supports Ubuntu')
-        codename = dist['codename']
+
+        id, _, _ = self.get_distro_info()
+        if id != "ubuntu": raise OSError('install_msbuild only supports ubuntu')
+        codename = distro.info()['codename']
 
         execute('curl https://packages.microsoft.com/keys/microsoft.asc | gpg --dearmor > /tmp/microsoft.gpg')
         execute('sudo mv /tmp/microsoft.gpg /etc/apt/trusted.gpg.d/microsoft.gpg')
