@@ -5,7 +5,7 @@ import os, sys, shutil, time
 from .types.dep_source import DepSource
 from .types.git import Git
 from .types.local_source import LocalSource
-from .utils.system import console
+from .utils.system import Color, console, error
 from .artifactory import artifactory_fetch_and_reconfigure
 from .util import normalized_join, normalized_path, write_text_to, read_lines_from
 from .parse_mamafile import parse_mamafile, update_mamafile_tag, update_cmakelists_tag
@@ -39,7 +39,7 @@ class BuildDependency:
         self.is_root = parent is None # Root deps are always built
         self.children: List[BuildDependency] = []
         self.product_sources = []
-        self.flattened_deps: List[BuildDependency] = [] # used for debugging
+        self.flattened_deps: List[BuildDependency] = [] # flat dependencies only, nothing else
 
         self.src_dir = None # source directory where the code is located
         self.dep_dir = None # dependency dir where platform build dirs are kept
@@ -86,6 +86,10 @@ class BuildDependency:
             raise RuntimeError(f'{self.name} src or git or pkg not configured. Specify at least one.')
 
 
+    def __str__(self): return f'BuildDependency {self.name} {self.dep_source}'
+    def __repr__(self): return f'BuildDependency {self.name} {self.dep_source}'
+
+
     @staticmethod
     def get_loaded_dependency(name:str) -> BuildDependency:
         if name in BuildDependency.loaded_deps:
@@ -112,12 +116,15 @@ class BuildDependency:
             # add new
             dep = BuildDependency(self, self.config, self.workspace, dep_source)
             BuildDependency.loaded_deps[dep_source.name] = dep
+            if self.config.verbose:
+                console(f'  - Target {self.name: <16} ADD {dep}', color=Color.BLUE)
 
         if dep in self.children:
             raise RuntimeError(f"BuildTarget {self.name} add dependency '{dep.name}'"\
                                 " failed because it has already been added")
 
         self.children.append(dep)
+        return dep
 
 
     def get_children(self) -> List[BuildDependency]:
@@ -208,44 +215,65 @@ class BuildDependency:
         return changed
 
 
-    def git_checkout(self):
-        # ignore non-git or root targets
-        if not self.dep_source.is_git or self.is_root:
-            return False
-        git:Git = self.dep_source
-        return git.dependency_checkout(self)
+    def _load_target(self) -> BuildTarget:
+        self.create_build_target() ## parses target mamafile
+        self.update_dep_dir() ## requires target mamafile workspace
+        self.create_build_dir_if_needed()
+        return self.target
 
 
     def _load(self):
-        git_changed = self.git_checkout()
-        self.create_build_target()  ## parses target mamafile
-        self.update_dep_dir()
-        self.create_build_dir_if_needed()
-
-        target = self.target
         conf = self.config
-        is_target = conf.target_matches(target.name)
-        if conf.clean and is_target:
-            self.clean()
+        if conf.verbose:
+            console(f'  - Target {self.name: <16} LOAD ({self.dep_source.get_type_string()})', color=Color.BLUE)
 
-        # load any build products from previous builds
-        if not self.is_root:
-            self.load_build_products(target)
+        is_target = conf.target_matches(self.name)
 
         # if this succeeds, it will overwrite products and libs
         # and sets self.from_artifactory
-        loaded_from_pkg = self.load_artifactory_package(target)
+        git_changed = False
+        loaded_from_pkg = False
+        should_load_art = self.should_load_artifactory()
+        if should_load_art and self.can_fetch_artifactory(print=True, which='LOAD'):
+            # for artifactory which is based on Git repo
+            if not self.is_root and self.dep_source.is_git:
+                git:Git = self.dep_source
+                git_changed = git.dependency_checkout(self)
+            target = self._load_target() # load target for artifactory
+            if conf.clean and is_target:
+                self.clean() ## requires a parsed mamafile target
+            fetched, dependencies = artifactory_fetch_and_reconfigure(target)
+            if fetched:
+                for dep_name in dependencies:
+                    self.add_child(dep_name)
+                loaded_from_pkg = True
+            elif self.dep_source.is_pkg:
+                raise RuntimeError(f'  - Target {self.name} failed to load artifactory pkg {self.dep_source}')
+        elif should_load_art and self.is_force_art_target():
+            raise RuntimeError(f'  - Target {self.name} failed to find artifactory pkg {self.dep_source} but `art` was specified')
+        else: # non-artifactory path:
+            target = self._load_target() # load target for Git and Src
+            if conf.clean and is_target:
+                self.clean() ## requires a parsed mamafile target
+            # load any build products from previous builds
+            if not self.is_root:
+                self.load_build_products(target)
+            if not self.is_root and self.dep_source.is_git:
+                git:Git = self.dep_source
+                git_changed = git.dependency_checkout(self)
+
+        if conf.verbose:
+            console(f'  - Target {self.name: <16} load settings and dependencies')
         target.settings() ## customization point for project settings
         target.dependencies() ## customization point for additional dependencies
 
-        if not loaded_from_pkg:
-            if self.is_root:
-                # fetch the compiler immediately from root settings
-                self.config.get_preferred_compiler_paths()
+        if not loaded_from_pkg and self.is_root:
+            # fetch the compiler immediately from root settings
+            conf.get_preferred_compiler_paths()
 
         build = False
         if conf.build or conf.update:
-            build = self._should_build(conf, target, is_target, git_changed)
+            build = self._should_build(conf, target, is_target, git_changed, loaded_from_pkg)
             if build:
                 self.create_build_dir_if_needed() # in case we just cleaned
             if git_changed:
@@ -259,19 +287,35 @@ class BuildDependency:
         return build
 
 
-    def load_artifactory_package(self, target):
-        # don't load anything during cleaning, because it will get cleaned anyways
-        load_art = not self.config.clean and \
-            (self.dep_source.is_pkg or os.path.exists(self.papa_package_file()) or self.is_first_time_build())
-        if load_art:
-            fetched, dependencies = artifactory_fetch_and_reconfigure(target)
-            if fetched:
-                for dep_name in dependencies:
-                    self.add_child(dep_name)
-                return True
-            elif self.dep_source.is_pkg:
-                raise RuntimeError(f'  - Target {target.name} failed to load artifactory pkg {self.dep_source}')
-        return False
+    def can_fetch_artifactory(self, print, which):
+        force_art = self.config.force_artifactory
+        disable_art = self.config.disable_artifactory
+        is_target = self.config.target_matches(self.name)
+
+        def noart(r):
+            if print and (self.config.print or force_art):
+                console(f'  - Target {self.name: <16} NO ARTIFACTORY PKG [{which} {r}]', color=Color.YELLOW)
+            return False
+
+        if disable_art:
+            return noart('noart override')
+        elif is_target and not force_art:
+            # don't load during rebuild -- defer to source based builds in that case
+            if self.config.rebuild: return noart('target rebuild')
+            # don't load anything during cleaning -- because it will get cleaned anyways
+            if self.config.clean: return noart('target clean')
+        elif print and (self.config.verbose or force_art):
+            console(f'  - Target {self.name: <16} CHECK ARTIFACTORY PKG [{which}]', color=Color.YELLOW)
+
+        return True
+
+    def is_force_art_target(self):
+        return not self.is_root and self.config.force_artifactory and self.config.target_matches(self.name)
+    
+    def should_load_artifactory(self):
+        should_load = self.dep_source.is_pkg or os.path.exists(self.papa_package_file()) or self.is_first_time_build()
+        is_force_art_target = self.is_force_art_target()
+        return should_load or is_force_art_target
 
 
     def _print_list(self, conf, target):
@@ -279,11 +323,11 @@ class BuildDependency:
             console(f'  - Target {target.name: <16}')
 
 
-    def _should_build(self, conf, target, is_target, git_changed):
+    def _should_build(self, conf:BuildConfig, target:BuildTarget, is_target, git_changed, loaded_from_pkg):
             def build(r):
                 if conf.print:
                     args = f'{target.args}' if target.args else ''
-                    console(f'  - Target {target.name: <16}   BUILD [{r}]  {args}')
+                    console(f'  - Target {target.name: <16} BUILD [{r}]  {args}', color=Color.YELLOW)
                 return True
 
             if conf.target and not is_target: # if we called: "target=SpecificProject"
@@ -309,11 +353,12 @@ class BuildDependency:
 
             # project has not defined `nothing_to_build` which is for header-only projects
             # thus we need to check if build should execute
-            can_build = not self.nothing_to_build
+            can_build = not loaded_from_pkg and not self.nothing_to_build
             if can_build:
                 # there are no build products defined at all, it hasn't been built or downloaded
                 if not target.build_products:
-                    if not self.has_build_files(): return build('not built yet')
+                    if not self.has_build_files():
+                        return build('not built yet')
                     return build('no build dependencies')
 
                 # we have build products, and none of them are missing
@@ -330,7 +375,7 @@ class BuildDependency:
                 if self.update_cmakelists_tag(): return build(target.name+'/CMakeLists.txt modified')
 
             if conf.print:
-                console(f'  - Target {target.name: <16}   OK')
+                console(f'  - Target {target.name: <16} OK', color=Color.GREEN)
             return False # do not build, all is ok
 
 
@@ -340,7 +385,7 @@ class BuildDependency:
             if first_changed and not self.should_rebuild:
                 self.should_rebuild = True
                 if self.config.print:
-                    console(f'  - Target {self.name: <16}   BUILD [{first_changed.name} changed]')
+                    console(f'  - Target {self.name: <16} BUILD [{first_changed.name} changed]')
                 self.create_build_dir_if_needed() # in case we just cleaned
 
 
@@ -360,9 +405,15 @@ class BuildDependency:
 
         # load the default mama.BuildTarget class
         mamaBuildTarget = getattr(sys.modules['mama.build_target'], 'BuildTarget')
+        mamaFilePath = self.mamafile_path()
+        if self.config.verbose:
+            exists = os.path.exists(mamaFilePath)
+            console(f'  - Target {self.name: <16} Load Mamafile: {mamaFilePath}  (Exists={exists})', color=Color.BLUE)
+            if not exists:
+                raise RuntimeError('omg')
 
         # this will load the specific `<class project(mama.build_target)>` class
-        project, buildTarget = parse_mamafile(self.config, mamaBuildTarget, self.mamafile_path())
+        project, buildTarget = parse_mamafile(self.config, mamaBuildTarget, mamaFilePath)
         if project and buildTarget:
             buildStatics = buildTarget.__dict__
             if not self.workspace:
@@ -380,6 +431,8 @@ class BuildDependency:
         else:
             if not self.workspace:
                 self.workspace = 'build'
+            if self.config.verbose:
+                console(f'  - Target {self.name: <16} Using Default BuildTarget Project={project} BuildTarget={buildTarget}', color=Color.YELLOW)
             self.target = mamaBuildTarget(name=self.name, config=self.config, dep=self, args=self.target_args)
 
 
@@ -413,11 +466,11 @@ class BuildDependency:
 
 
     def update_mamafile_tag(self):
-        return self.src_dir and update_mamafile_tag(self.mamafile_path(), self.build_dir)
+        return self.src_dir and update_mamafile_tag(self.config, self.mamafile_path(), self.build_dir)
 
 
     def update_cmakelists_tag(self):
-        return self.src_dir and update_cmakelists_tag(self.cmakelists_path(), self.build_dir)
+        return self.src_dir and update_cmakelists_tag(self.config, self.cmakelists_path(), self.build_dir)
 
 
     def build_file_exists(self, filename):
@@ -483,11 +536,11 @@ class BuildDependency:
     ## Clean
     def clean(self):
         if self.config.print:
-            console(f'  - Target {self.name: <16}   CLEAN  {self.config.platform_build_dir_name()}')
+            console(f'  - Target {self.name: <16} CLEAN  {self.config.platform_build_dir_name()}')
 
         if self.build_dir == '/' or not os.path.exists(self.build_dir):
             return
-        
+
         self.target.clean() # Customization point
         shutil.rmtree(self.build_dir, ignore_errors=True)
 
