@@ -1,23 +1,26 @@
 from __future__ import annotations
 from typing import TYPE_CHECKING
 import os
-from .utils.system import System, console
-from .utils.sub_process import SubProcess, execute
+from .utils.system import System, console, Color
+from .utils.sub_process import SubProcess, execute, execute_piped_echo
 
 if TYPE_CHECKING:
     from .build_target import BuildTarget
     from .build_config import BuildConfig
 
 
-def _rerunnable_cmake_conf(cmd, cwd, allow_rerun, target:BuildTarget):
+def _rerunnable_cmake_conf(cmd, cwd, allow_rerun, target:BuildTarget, delete_cmakecache:bool = False):
     rerun = False
     error = ''
-    delete_cmakecache = False
     print_enabled = target.config.print
     verbose = target.config.verbose
     if verbose: console(cmd)
     #xcode_filter = (target.ios or target.macos) and not target.enable_ninja_build 
 
+    if delete_cmakecache:
+        if print_enabled: console('Deleting CMakeCache.txt')
+        os.remove(target.build_dir('CMakeCache.txt'))
+
     def handle_output(line:str):
         nonlocal rerun, delete_cmakecache
         print(line) # newline is not included
@@ -32,19 +35,22 @@ def _rerunnable_cmake_conf(cmd, cwd, allow_rerun, target:BuildTarget):
             rerun = True
             delete_cmakecache = True
 
+    # run CMake configure and handle output
     exit_status = SubProcess.run(cmd, cwd, io_func=handle_output)
 
-    if delete_cmakecache:
-        if print_enabled: console('Deleting CMakeCache.txt')
-        os.remove(target.build_dir('CMakeCache.txt'))
     if rerun and allow_rerun:
         if print_enabled: console('Rerunning CMake configure')
-        return _rerunnable_cmake_conf(cmd, cwd, False, target)
+        return _rerunnable_cmake_conf(cmd, cwd, False, target, delete_cmakecache=delete_cmakecache)
     if exit_status != 0:
         raise Exception(f'CMake configure error: {error}')
 
 
 def run_config(target:BuildTarget):
+    if not target.config.update and os.path.exists(target.build_dir('CMakeCache.txt')):
+        if target.config.verbose:
+            console('Not running CMake configure because CMakeCache.txt exists and `update` was not specified')
+        return
+
     def get_flags():
         flags = ''
         options = target.cmake_opts + _default_options(target) + target.get_product_defines()
@@ -54,15 +60,38 @@ def run_config(target:BuildTarget):
     cmake_flags = get_flags()
     generator = _generator(target)
     src_dir = target.source_dir()
-    cmd = f'cmake {generator} {cmake_flags} -DCMAKE_INSTALL_PREFIX="." "{src_dir}"'
+    install_prefix = '-DCMAKE_INSTALL_PREFIX="."'
+    # # use install prefix override for libraries, but for root target, leave it open-ended
+    # install_prefix = '' if target.dep.is_root else '-DCMAKE_INSTALL_PREFIX="."'
+    cmd = f'cmake {generator} {cmake_flags} {install_prefix} "{src_dir}"'
     _rerunnable_cmake_conf(cmd, target.build_dir(), True, target)
 
 
-def run_build(target:BuildTarget, install:bool, extraflags=''):
+def is_rerunnable_error(output:str):
+    """ Checks output string if a rerunnable error occurred. 
+        These are non-fatal errors that disappear with a simple cmake configure. """
+    return 'Makefile: No such file or directory' in output
+
+
+def run_build(target:BuildTarget, install:bool, extraflags='', rerun=True):
     build_dir = target.build_dir()
     flags = _build_config(target, install)
     extraflags = _buildsys_flags(target)
-    execute(f'cmake --build {build_dir} {flags} {extraflags}', echo=target.config.verbose)
+    cmd = f'cmake --build {build_dir} {flags} {extraflags}'
+    if target.config.verbose:
+        console(cmd, color=Color.GREEN)
+    status, output = execute_piped_echo(build_dir, cmd, echo=True)
+    if status != 0:
+        if rerun and is_rerunnable_error(output):
+            if target.config.verbose:
+                console(f'Build {target.name} failed, attempting to rerun config', color=Color.GREEN)
+            cmake_cache = target.build_dir('CMakeCache.txt')
+            if os.path.exists(cmake_cache):
+                os.remove(cmake_cache)
+            run_config(target)
+            run_build(target, install, extraflags, rerun=False)
+        else:
+            raise Exception(f'{cmd} failed with return code {status}')
 
 
 def _generator(target:BuildTarget):
@@ -159,9 +188,7 @@ def _default_options(target:BuildTarget):
         for path in config.raspi_includes():
             add_flag(f'-I {path}')
     elif config.oclea:
-        add_flag('--sysroot', config.oclea_sysroot())
-        for path in config.oclea_includes():
-            add_flag(f'-I {path}')
+        config.oclea.get_cxx_flags(add_flag)
 
     if config.flags:
         add_flag(config.flags)
@@ -230,19 +257,7 @@ def _default_options(target:BuildTarget):
             if config.print: console(f'Toolchain: {toolchain}')
             opt += [f'CMAKE_TOOLCHAIN_FILE="{toolchain}"']
     elif config.oclea:
-        opt += [
-            'OCLEA=TRUE',
-            'CMAKE_SYSTEM_NAME=Linux',
-            'CMAKE_SYSTEM_VERSION=1',
-            'CMAKE_SYSTEM_PROCESSOR=arm64',
-            'CMAKE_FIND_ROOT_PATH_MODE_PROGRAM=NEVER', # Use our definitions for compiler tools
-            'CMAKE_FIND_ROOT_PATH_MODE_LIBRARY=ONLY', # Search for libraries and headers in the target directories only
-            'CMAKE_FIND_ROOT_PATH_MODE_INCLUDE=ONLY',
-        ]
-        if target.cmake_oclea_toolchain:
-            toolchain = target.source_dir(target.cmake_oclea_toolchain)
-            if config.print: console(f'Toolchain: {toolchain}')
-            opt += [f'CMAKE_TOOLCHAIN_FILE="{toolchain}"']
+        opt += config.oclea.get_cmake_build_opts()
     elif config.macos:
         pass
     elif config.ios:
