@@ -2,13 +2,13 @@
 import sys, os
 
 from .types.local_source import LocalSource
-from .utils.system import console
+from .utils.system import Color, console
 from .utils.sub_process import execute
 from .util import glob_with_extensions, glob_folders_with_name_match
 from .build_config import BuildConfig
 from .build_target import BuildTarget
 from .build_dependency import BuildDependency
-from .dependency_chain import load_dependency_chain, execute_task_chain, find_dependency, get_full_flattened_deps
+from .dependency_chain import load_dependency_chain, execute_task_chain, find_dependency, get_flat_deps
 from .init_project import mama_init_project
 
 def print_title():
@@ -29,6 +29,8 @@ def print_usage():
     console('    wipe       - alias of reclone')
     console('    upload     - uploads target package to artifactory server')
     console('    if_needed  - only uploads if package does not exist on server')
+    console('    art        - always fetch pkgs from artifactory, failure will throw an error')
+    console('    noart      - temporarily ignore artifactory pkgs fetch')
     console('    test       - run tests for main project or specific target')
     console('    start=arg  - start a specific tool via mamafile.start(args)')
     console('    add        - add new dependency')
@@ -133,6 +135,15 @@ def check_config_target(config: BuildConfig, root: BuildDependency):
             exit(-1)
 
 
+def print_package_exports(dep: BuildDependency):
+    target:BuildTarget = dep.target
+    if dep.from_artifactory or target.try_automatic_artifactory_fetch():
+        console(f'    Target {target.name} fetched from artifactory')
+    else:
+        target.package()
+    target.print_exports(abs_paths=True)
+
+
 def main():
     if sys.version_info < (3, 6):
         console('FATAL ERROR: MamaBuild requires Python 3.6')
@@ -196,24 +207,26 @@ def main():
         mama_init_project(dep)
         return
 
+    flat_deps = get_flat_deps(root)
+    flat_deps_reverse = list(reversed(flat_deps))
+
     if config.list:
-        console(f'    Dependency List: {get_full_flattened_deps(root)}')
-        if config.target:
-            dep = find_dependency(root, config.target)
-            if dep:
-                target:BuildTarget = dep.target
-                if dep.from_artifactory or target.try_automatic_artifactory_fetch():
-                    console(f'    Target {target.name} fetched from artifactory')
-                else:
-                    target.package()
-                target.print_exports(abs_paths=True)
+        dep = find_dependency(root, config.target) if config.target else root
+        flat_deps_names = [d.name for d in flat_deps]
+        console(f'    {dep.name} Dependency List: {flat_deps_names}', Color.BLUE)
+        print_package_exports(dep)
         return
 
-    if config.android: config.init_ndk_path()
-    if config.raspi:   config.init_raspi_path()
-    if config.oclea:   config.init_oclea_path()
+    if config.android: config.android_home()
+    if config.raspi:   config.raspi_bin()
+    if config.oclea:   config.oclea.bin()
 
-    execute_task_chain(root)
+    if config.verbose:
+        chain = ' -> '.join([d.name for d in flat_deps_reverse])
+        console(f'Executing task chain for build:\n    {chain}', Color.BLUE)
+        print_package_exports(root)
+
+    execute_task_chain(flat_deps_reverse)
 
     if config.open:
         open_project(config, root)
